@@ -119,7 +119,9 @@ static void addressbook_list_selected		(GtkCList	*clist,
 						 gint		 column,
 						 GdkEvent	*event,
 						 gpointer	 data);
+#if 0
 static void addressbook_entry_changed		(GtkWidget	*widget);
+#endif
 
 static void addressbook_list_button_pressed	(GtkWidget	*widget,
 						 GdkEventButton	*event,
@@ -413,8 +415,13 @@ static void addressbook_create(void)
 
 	entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+
+	address_completion_register_entry(GTK_ENTRY(entry));
+
+#if 0
 	gtk_signal_connect(GTK_OBJECT(entry), "changed",
 			   GTK_SIGNAL_FUNC(addressbook_entry_changed), NULL);
+#endif
 
 	paned = gtk_hpaned_new();
 	gtk_box_pack_start(GTK_BOX(vbox2), paned, TRUE, TRUE, 0);
@@ -517,6 +524,8 @@ static void addressbook_create(void)
 	addrbook.tree_factory = tree_factory;
 	addrbook.list_factory = list_factory;
 
+	address_completion_start(window);
+
 	gtk_widget_show_all(window);
 }
 
@@ -775,9 +784,11 @@ static void addressbook_list_selected(GtkCList *clist, gint row, gint column,
 		return;
 	}
 
+#if 0
 	gtk_signal_handler_block_by_func
 		(GTK_OBJECT(entry),
 		 GTK_SIGNAL_FUNC(addressbook_entry_changed), NULL);
+#endif		 
 
 	gtk_entry_set_text(entry, "");
 
@@ -807,11 +818,14 @@ static void addressbook_list_selected(GtkCList *clist, gint row, gint column,
 		}
 	}
 
+#if 0
 	gtk_signal_handler_unblock_by_func
 		(GTK_OBJECT(entry),
 		 GTK_SIGNAL_FUNC(addressbook_entry_changed), NULL);
+#endif		 
 }
 
+#if 0
 static void addressbook_entry_changed(GtkWidget *widget)
 {
 	GtkCList *clist = GTK_CLIST(addrbook.clist);
@@ -851,6 +865,7 @@ static void addressbook_entry_changed(GtkWidget *widget)
 
 	gtk_clist_unselect_all(clist);
 }
+#endif
 
 static void addressbook_list_button_pressed(GtkWidget *widget,
 					    GdkEventButton *event,
