@@ -21,17 +21,11 @@
 #  include "config.h"
 #endif
 
-#undef USE_ICONV
-
 #include <glib.h>
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
 
-#ifdef USE_ICONV
-#  include <iconv.h>
-#endif /* USE_ICONV */
-
 #if (HAVE_WCTYPE_H && HAVE_WCHAR_H)
 #  include <wchar.h>
 #  include <wctype.h>
@@ -63,34 +57,11 @@
 
 void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-#ifdef USE_ICONV
-	static iconv_t cd;
-	size_t inlen;
-
-	if (!cd)
-		cd = iconv_open(CS_EUC_JP, CS_ISO_2022_JP);
-
-	inlen = strlen(inbuf) + 1;
-	memset(outbuf, 0, inlen);
-	iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
-#else
 	KCC_filter(outbuf, "EUC", (gchar *)inbuf, "JISBB", 0, 0, 0);
-#endif /* USE_ICONV */
 }
 
 void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-#ifdef USE_ICONV
-	static iconv_t cd;
-	size_t inlen;
-
-	if (!cd)
-		cd = iconv_open(CS_ISO_2022_JP, CS_EUC_JP);
-
-	inlen = strlen(inbuf) + 1;
-	memset(outbuf, 0, MIN(inlen * 3, outlen));
-	iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
-#else
 	size_t inlen, len;
 
 	inlen = strlen(inbuf);
@@ -126,7 +97,6 @@ void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 		strncpy2(outbuf, tmpout, outlen);
 	} else
 		KCC_filter(outbuf, "JISBB", (gchar *)inbuf, "EUC", 0, 0, 0);
-#endif /* USE_ICONV */
 }
 
 void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
@@ -311,6 +281,46 @@ void conv_noconv(gchar *outbuf, gint outlen, const gchar *inbuf)
 	strncpy2(outbuf, inbuf, outlen);
 }
 
+CodeConverter *conv_code_converter_new(const gchar *charset)
+{
+	CodeConverter *conv;
+
+	conv = g_new0(CodeConverter, 1);
+#if !HAVE_LIBJCONV
+	conv->code_conv_func = conv_get_code_conv_func(charset);
+#endif
+	conv->charset_str = g_strdup(charset);
+	conv->charset = conv_get_charset_from_str(charset);
+
+	return conv;
+}
+
+void conv_code_converter_destroy(CodeConverter *conv)
+{
+	g_free(conv->charset_str);
+	g_free(conv);
+}
+
+gint conv_convert(CodeConverter *conv, gchar *outbuf, gint outlen,
+		  const gchar *inbuf)
+{
+#if HAVE_LIBJCONV
+	gchar *str;
+
+	str = conv_codeset_strdup(inbuf, conv->charset_str, NULL);
+	if (!str)
+		return -1;
+	else {
+		strncpy2(outbuf, str, outlen);
+		g_free(str);
+	}
+#else /* !HAVE_LIBJCONV */
+	conv->code_conv_func(outbuf, outlen, inbuf);
+#endif
+
+	return 0;
+}
+
 gchar *conv_codeset_strdup(const gchar *inbuf,
 			   const gchar *src_codeset, const gchar *dest_codeset)
 {
@@ -556,6 +566,8 @@ CharSet conv_get_charset_from_str(const gchar *charset)
 {
 	gint i;
 
+	if (!charset) return C_AUTO;
+
 	for (i = 0; i < sizeof(charsets) / sizeof(charsets[0]); i++) {
 		if (!strcasecmp(charsets[i].name, charset))
 			return charsets[i].charset;
