@@ -161,13 +161,17 @@ struct _AttachInfo
 	off_t size;
 };
 
-void compose_new	(PrefsAccount	*account);
-void compose_reply	(MsgInfo	*msginfo,
-			 gboolean	 quote,
-			 gboolean	 to_all);
-void compose_forward	(MsgInfo	*msginfo,
-			 gboolean	 as_attach);
-void compose_reedit	(MsgInfo	*msginfo);
+void compose_new		(PrefsAccount	*account);
+
+void compose_new_with_recipient	(PrefsAccount	*account,
+				 const gchar	*to);
+
+void compose_reply		(MsgInfo	*msginfo,
+				 gboolean	 quote,
+				 gboolean	 to_all);
+void compose_forward		(MsgInfo	*msginfo,
+				 gboolean	 as_attach);
+void compose_reedit		(MsgInfo	*msginfo);
 
 GList *compose_get_compose_list	(void);
 
