@@ -648,10 +648,14 @@ static void folder_init(Folder *folder, FolderType type, const gchar *name)
 		folder->remove_folder       = mh_remove_folder;
 		break;
 	case F_IMAP:
-		folder->get_msg_list  = imap_get_msg_list;
-		folder->fetch_msg     = imap_fetch_msg;
-		folder->scan          = imap_scan_folder;
-		folder->create_folder = imap_create_folder;
+		folder->get_msg_list        = imap_get_msg_list;
+		folder->fetch_msg           = imap_fetch_msg;
+		folder->move_msg            = imap_move_msg;
+		folder->move_msgs_with_dest = imap_move_msgs_with_dest;
+		folder->remove_msg          = imap_remove_msg;
+		folder->remove_all_msg      = imap_remove_all_msg;
+		folder->scan                = imap_scan_folder;
+		folder->create_folder       = imap_create_folder;
 		break;
 	case F_NEWS:
 		folder->get_msg_list = news_get_article_list;
