@@ -1515,7 +1515,7 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		if (item != NULL &&
 		    item->path != NULL &&
 		    folderview->summaryview->folder_item != item) {
-			if (item->folder->type == F_MH ) {
+			if (item->folder->type != F_NEWS) {
 				gtk_ctree_select(GTK_CTREE(widget), node);
 				gdk_drag_status(context,
 						context->suggested_action,
