@@ -224,3 +224,22 @@ void gtkut_widget_wait_for_draw(GtkWidget *widget)
 	while (!flag)
 		gtk_main_iteration();
 }
+
+void gtkut_widget_get_uposition(GtkWidget *widget, gint *px, gint *py)
+{
+	gint x, y;
+	gint sx, sy;
+
+	g_return_if_fail(widget != NULL);
+	g_return_if_fail(widget->window != NULL);
+
+	/* gdk_window_get_root_origin ever return *rootwindow*'s position*/
+	gdk_window_get_root_origin(widget->window, &x, &y);
+
+	sx = gdk_screen_width();
+	sy = gdk_screen_height();
+	x %= sx; if (x < 0) x += sx;
+	y %= sy; if (y < 0) y += sy;
+	*px = x;
+	*py = y;
+}
