@@ -48,38 +48,109 @@ static GSList *imap_get_uncached_messages(IMAPSession	*session,
 					  FolderItem	*item,
 					  gint		first,
 					  gint		last);
-static GSList *imap_delete_old_messages	 (GSList	*mlist,
-					  gint		 first);
+static GSList *imap_delete_messages	 (GSList	*mlist,
+					  gint		first,
+					  gint		last);
 static void imap_delete_all_messages	 (FolderItem	*item);
 
-static gint imap_open(const gchar *server, gushort port, gchar *buf);
-static gint imap_auth(gint sock, const gchar *user, const gchar *pass);
-static gint imap_logout(gint sock);
-static gint imap_noop(gint sock);
-static gint imap_select(gint sock, const gchar *folder,
-			gint *exists, gint *recent, gint *unseen, gulong *uid);
-static gint imap_get_envelope(gint sock, gint first, gint last);
-static gint imap_get_message(gint sock, gint num, const gchar *filename);
+static gint imap_open		(const gchar	*server,
+				 gushort	 port,
+				 gchar		*buf);
+static gint imap_auth		(gint		 sock,
+				 const gchar	*user,
+				 const gchar	*pass);
+static gint imap_logout		(gint		 sock);
+static gint imap_noop		(gint		 sock);
+static gint imap_select		(gint		 sock,
+				 const gchar	*folder,
+				 gint		*exists,
+				 gint		*recent,
+				 gint		*unseen,
+				 gulong		*uid);
+static gint imap_get_envelope	(gint		 sock,
+				 gint		 first,
+				 gint		 last);
+#if 0
+static gint imap_search		(gint		 sock,
+				 GSList		*numlist);
+#endif
+static gint imap_get_message	(gint		 sock,
+				 gint		 num,
+				 const gchar	*filename);
+static gint imap_copy_message	(gint		 sock,
+				 gint		 num,
+				 const gchar	*destfolder);
+static gint imap_store		(gint		 sock,
+				 gint		 first,
+				 gint		 last,
+				 gchar		*sub_cmd);
 
-static gchar *imap_parse_atom(gint sock, gchar *src, gchar *dest,
-			      gchar *orig_buf);
-static gchar *imap_parse_one_address(gint sock, gchar *start,
-				     gchar *out_from_str,
-				     gchar *out_fromname_str,
-				     gchar *orig_buf);
-static gchar *imap_parse_address(gint sock, gchar *start,
-				 gchar **out_from_str,
-				 gchar **out_fromname_str,
-				 gchar *orig_buf);
-static MsgFlags imap_parse_flags(const gchar *flag_str);
-static MsgInfo *imap_parse_envelope(gint sock, gchar *line_str);
+static gint imap_set_article_flags	(IMAPSession	*session,
+					 gint		 first,
+					 gint		 last,
+					 IMAPFlags	 flag,
+					 gboolean	 is_set);
+static gint imap_expunge		(IMAPSession	*session);
 
-static gint imap_ok(gint sock, GPtrArray *argbuf);
-static void imap_gen_send(gint sock, const gchar *format, ...);
-static gint imap_gen_recv(gint sock, gchar *buf, gint size);
+static gchar *imap_parse_atom		(gint	 sock,
+					 gchar	*src,
+					 gchar	*dest,
+					 gchar	*orig_buf);
+static gchar *imap_parse_one_address	(gint	 sock,
+					 gchar	*start,
+					 gchar	*out_from_str,
+					 gchar	*out_fromname_str,
+					 gchar	*orig_buf);
+static gchar *imap_parse_address	(gint	 sock,
+					 gchar	*start,
+					 gchar **out_from_str,
+					 gchar **out_fromname_str,
+					 gchar	*orig_buf);
+static MsgFlags imap_parse_flags	(const gchar	*flag_str);
+static MsgInfo *imap_parse_envelope	(gint	 sock,
+					 gchar	*line_str);
 
-static gchar *search_array_contain_str(GPtrArray *array, gchar *str);
+static gint imap_ok		(gint		 sock,
+				 GPtrArray	*argbuf);
+static void imap_gen_send	(gint		 sock,
+				 const gchar	*format, ...);
+static gint imap_gen_recv	(gint		 sock,
+				 gchar		*buf,
+				 gint		 size);
 
+static gchar *search_array_contain_str	(GPtrArray	*array,
+					 gchar		*str);
+static void imap_path_subst_slash_to_dot(gchar *str);
+
+
+static IMAPSession *imap_session_connect_if_not(Folder *folder)
+{
+	RemoteFolder *rfolder = REMOTE_FOLDER(folder);
+
+	g_return_val_if_fail(folder != NULL, NULL);
+	g_return_val_if_fail(folder->type == F_IMAP, NULL);
+	g_return_val_if_fail(folder->account != NULL, NULL);
+
+	if (!rfolder->session) {
+		rfolder->session =
+			imap_session_new(folder->account->recv_server, 143,
+					 folder->account->userid,
+					 folder->account->passwd);
+	} else {
+		if (imap_noop(rfolder->session->sock) != IMAP_SUCCESS) {
+			log_warning(_("IMAP4 connection to %s:%d has been"
+				      " disconnected. Reconnecting...\n"),
+				    folder->account->recv_server, 143);
+			session_destroy(rfolder->session);
+			rfolder->session =
+				imap_session_new(folder->account->recv_server,
+						 143, folder->account->userid,
+						 folder->account->passwd);
+		}
+	}
+
+	return IMAP_SESSION(rfolder->session);
+}
 
 Session *imap_session_new(const gchar *server, gushort port,
 			  const gchar *user, const gchar *pass)
@@ -144,25 +215,7 @@ GSList *imap_get_msg_list(Folder *folder, FolderItem *item, gboolean use_cache)
 	g_return_val_if_fail(folder->type == F_IMAP, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
 
-	if (!REMOTE_FOLDER(folder)->session) {
-		REMOTE_FOLDER(folder)->session =
-			imap_session_new(folder->account->recv_server, 143,
-					 folder->account->userid,
-					 folder->account->passwd);
-	} else {
-		if (imap_noop(REMOTE_FOLDER(folder)->session->sock)
-		    != IMAP_SUCCESS) {
-			log_warning(_("IMAP4 connection to %s:%d has been"
-				      " disconnected. Reconnecting...\n"),
-				    folder->account->recv_server, 143);
-			session_destroy(REMOTE_FOLDER(folder)->session);
-			REMOTE_FOLDER(folder)->session =
-				imap_session_new(folder->account->recv_server,
-						 143, folder->account->userid,
-						 folder->account->passwd);
-		}
-	}
-	session = IMAP_SESSION(REMOTE_FOLDER(folder)->session);
+	session = imap_session_connect_if_not(folder);
 
 	if (!session) {
 		mlist = procmsg_read_cache(item, FALSE);
@@ -216,7 +269,7 @@ GSList *imap_get_msg_list(Folder *folder, FolderItem *item, gboolean use_cache)
 
 			newlist = imap_get_uncached_messages
 				(session, item, begin, exists);
-			imap_delete_old_messages(mlist, begin);
+			imap_delete_messages(mlist, begin, INT_MAX);
 			mlist = g_slist_concat(mlist, newlist);
 		} else if (begin == 1) {
 			mlist = imap_get_uncached_messages 
@@ -235,11 +288,14 @@ GSList *imap_get_msg_list(Folder *folder, FolderItem *item, gboolean use_cache)
 gchar *imap_fetch_msg(Folder *folder, FolderItem *item, gint num)
 {
 	gchar *path, *filename;
+	IMAPSession *session;
 	gint ok;
 
 	g_return_val_if_fail(folder != NULL, NULL);
 	g_return_val_if_fail(item != NULL, NULL);
 
+	session = imap_session_connect_if_not(folder);
+
 	path = folder_item_get_path(item);
 	filename = g_strconcat(path, G_DIR_SEPARATOR_S, itos(num), NULL);
 	g_free(path);
@@ -249,15 +305,16 @@ gchar *imap_fetch_msg(Folder *folder, FolderItem *item, gint num)
 		return filename;
 	}
 
-	if (!REMOTE_FOLDER(folder)->session) {
+	if (!session) {
 		g_free(filename);
 		return NULL;
 	}
 
 	debug_print(_("getting message %d...\n"), num);
-	ok = imap_get_message(REMOTE_FOLDER(folder)->session->sock,
-			      num, filename);
+	ok = imap_get_message(SESSION(session)->sock, num, filename);
+
 	statusbar_pop_all();
+
 	if (ok != IMAP_SUCCESS) {
 		g_warning(_("can't fetch message %d\n"), num);
 		g_free(filename);
@@ -267,6 +324,155 @@ gchar *imap_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	return filename;
 }
 
+gint imap_move_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
+{
+	gchar *destdir;
+	IMAPSession *session;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msginfo != NULL, -1);
+
+	session = imap_session_connect_if_not(folder);
+	if (!session) return -1;
+
+	if (msginfo->folder == dest) {
+		g_warning(_("the src folder is identical to the dest.\n"));
+		return -1;
+	}
+
+	Xstrdup_a(destdir, dest->path, return -1);
+	imap_path_subst_slash_to_dot(destdir);
+
+	imap_copy_message(SESSION(session)->sock, msginfo->msgnum, destdir);
+	imap_set_article_flags(session, msginfo->msgnum, msginfo->msgnum,
+			       IMAP_FLAG_DELETED, TRUE);
+	debug_print(_("Moving message %s%c%d to %s ...\n"),
+		    msginfo->folder->path, G_DIR_SEPARATOR,
+		    msginfo->msgnum, dest->path);
+
+	imap_expunge(session);
+
+	return IMAP_SUCCESS;
+}
+
+gint imap_move_msgs_with_dest(Folder *folder, FolderItem *dest, 
+			      GSList *msglist)
+{
+	gchar *destdir;
+	GSList *cur;
+	MsgInfo *msginfo;
+	IMAPSession *session;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(msglist != NULL, -1);
+
+	session = imap_session_connect_if_not(folder);
+	if (!session) return -1;
+
+	Xalloca(destdir, strlen(dest->path) + 1, return -1);
+	strcpy(destdir, dest->path);
+	imap_path_subst_slash_to_dot(destdir);
+
+	for (cur = msglist; cur != NULL; cur = cur->next) {
+		msginfo = (MsgInfo *)cur->data;
+
+		if (msginfo->folder == dest) {
+			g_warning(_("the src folder is identical to the dest.\n"));
+			continue;
+		}
+		imap_copy_message(SESSION(session)->sock,
+				  msginfo->msgnum, destdir);
+		imap_set_article_flags
+			(session, msginfo->msgnum, msginfo->msgnum,
+			 IMAP_FLAG_DELETED, TRUE);
+
+		debug_print(_("Moving message %s%c%d to %s ...\n"),
+			    msginfo->folder->path, G_DIR_SEPARATOR,
+			    msginfo->msgnum, dest->path);
+		
+	}
+
+	imap_expunge(session);
+
+	return IMAP_SUCCESS;
+
+}
+
+gint imap_remove_msg(Folder *folder, FolderItem *item, gint num)
+{
+	gint exists, recent, unseen;
+	gulong uid;
+	gint ok;
+	IMAPSession *session;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(num > 0 && num <= item->last_num, -1);
+
+	session = imap_session_connect_if_not(folder);
+	if (!session) return -1;
+
+	ok = imap_select(SESSION(session)->sock, item->path,
+			 &exists, &recent, &unseen, &uid);
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't select folder: %s\n"), item->path);
+		return ok;
+	}
+
+	ok = imap_set_article_flags(IMAP_SESSION(REMOTE_FOLDER(folder)->session),
+				    num, num, IMAP_FLAG_DELETED, TRUE);
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't set deleted flags: %d\n"), num);
+		return ok;
+	}
+
+	ok = imap_expunge(session);
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't expunge\n"));
+		return ok;
+	}
+
+	return IMAP_SUCCESS;
+}
+
+gint imap_remove_all_msg(Folder *folder, FolderItem *item)
+{
+	gint exists, recent, unseen;
+	gulong uid;
+	gint ok;
+	IMAPSession *session;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(item != NULL, -1);
+
+	session = imap_session_connect_if_not(folder);
+	if (!session) return -1;
+
+	ok = imap_select(SESSION(session)->sock, item->path,
+			 &exists, &recent, &unseen, &uid);
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't select folder: %s\n"), item->path);
+		return ok;
+	}
+
+	ok = imap_set_article_flags(session, 1, exists,
+				    IMAP_FLAG_DELETED, TRUE);
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't set deleted flags: 1:%d\n"), exists);
+		return ok;
+	}
+
+	ok = imap_expunge(session);
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't expunge\n"));
+		return ok;
+	}
+
+	return IMAP_SUCCESS;
+}
+
 void imap_scan_folder(Folder *folder, FolderItem *item)
 {
 }
@@ -281,6 +487,8 @@ FolderItem *imap_create_folder(Folder *folder, FolderItem *parent,
 	g_return_val_if_fail(parent != NULL, NULL);
 	g_return_val_if_fail(name != NULL, NULL);
 
+	imap_session_connect_if_not(folder);
+
 	if (parent->path)
 		path = g_strconcat(parent->path, G_DIR_SEPARATOR_S, name,
 				   NULL);
@@ -342,7 +550,7 @@ static GSList *imap_get_uncached_messages(IMAPSession *session,
 	return newlist;
 }
 
-static GSList *imap_delete_old_messages(GSList *mlist, gint first)
+static GSList *imap_delete_messages(GSList *mlist, gint first, gint last)
 {
 	GSList *cur, *next;
 	MsgInfo *msginfo;
@@ -352,7 +560,9 @@ static GSList *imap_delete_old_messages(GSList *mlist, gint first)
 		next = cur->next;
 
 		msginfo = (MsgInfo *)cur->data;
-		if (msginfo && msginfo->msgnum >= first) {
+		if ((msginfo) && 
+		    (first <= msginfo->msgnum ) &&
+		    (msginfo->msgnum <= last)) {
 			debug_print(_("deleting message %d...\n"),
 				    msginfo->msgnum);
 
@@ -466,11 +676,15 @@ static gint imap_select(gint sock, const gchar *folder,
 	gint ok;
 	gchar *resp_str;
 	GPtrArray *argbuf;
+	gchar *imappath;
 
 	*exists = *recent = *unseen = *uid = 0;
 	argbuf = g_ptr_array_new();
 
-	imap_gen_send(sock, "SELECT \"%s\"", folder);
+	Xalloca(imappath, strlen(folder) + 1, return -1);
+	strcpy(imappath, folder);
+	imap_path_subst_slash_to_dot(imappath);
+	imap_gen_send(sock, "SELECT \"%s\"", imappath);
 	if ((ok = imap_ok(sock, argbuf)) != IMAP_SUCCESS)
 		goto bail;
 
@@ -569,6 +783,22 @@ static gint imap_get_message(gint sock, gint num, const gchar *filename)
 	return ok;
 }
 
+static gint imap_copy_message(gint sock, gint num, const gchar *destfolder)
+{
+	gint ok;
+
+	g_return_val_if_fail(destfolder != NULL, IMAP_ERROR);
+
+	imap_gen_send(sock, "COPY %d %s", num, destfolder);
+	ok = imap_ok(sock, NULL);
+	if (ok != IMAP_SUCCESS) {
+		log_warning(_("can't copy %d to %s\n"), num, destfolder);
+		return -1;
+	}
+
+	return ok;
+}
+
 static gchar *imap_parse_atom(gint sock, gchar *src, gchar *dest,
 			      gchar *orig_buf)
 {
@@ -858,6 +1088,75 @@ gint imap_get_envelope(gint sock, gint first, gint last)
 	return IMAP_SUCCESS;
 }
 
+static gint imap_store(gint sock, gint first, gint last, gchar *sub_cmd)
+{
+	gint ok;
+	GPtrArray *argbuf;
+
+	argbuf = g_ptr_array_new();
+
+	imap_gen_send(sock, "STORE %d:%d %s", first, last, sub_cmd);
+	
+	if ((ok = imap_ok(sock, argbuf)) != IMAP_SUCCESS) {
+		g_ptr_array_free(argbuf, TRUE);
+		log_warning(_("error while imap command: STORE %d:%d %s\n"),
+			    first, last, sub_cmd);
+		return ok;
+	}
+
+	g_ptr_array_free(argbuf, TRUE);
+
+	return IMAP_SUCCESS;
+}
+
+static gint imap_set_article_flags(IMAPSession *session,
+				   gint first,
+				   gint last,
+				   IMAPFlags flags,
+				   gboolean is_set)
+{
+	GString *buf;
+	gint ok;
+
+	buf = g_string_new(is_set ? "+FLAGS (" : "-FLAGS (");
+
+	if (IMAP_IS_SEEN(flags))	g_string_append(buf, "\\Seen ");
+	if (IMAP_IS_ANSWERED(flags))	g_string_append(buf, "\\Answered ");
+	if (IMAP_IS_FLAGGED(flags))	g_string_append(buf, "\\Flagged ");
+	if (IMAP_IS_DELETED(flags))	g_string_append(buf, "\\Deleted ");
+	if (IMAP_IS_DRAFT(flags))	g_string_append(buf, "\\Draft");
+
+	if (buf->str[buf->len - 1] == ' ')
+		g_string_truncate(buf, buf->len - 1);
+
+	g_string_append_c(buf, ')');
+
+	ok = imap_store(SESSION(session)->sock, first, last, buf->str);
+	g_string_free(buf, TRUE);
+
+	return ok;
+}
+
+static gint imap_expunge(IMAPSession *session)
+{
+	gint ok;
+	GPtrArray *argbuf;
+
+	argbuf = g_ptr_array_new();
+
+	imap_gen_send(SESSION(session)->sock, "EXPUNGE");
+
+	if ((ok = imap_ok(SESSION(session)->sock, argbuf)) != IMAP_SUCCESS) {
+		log_warning(_("error while imap command: EXPUNGE\n"));
+		g_ptr_array_free(argbuf, TRUE);
+		return ok;
+	}
+
+	g_ptr_array_free(argbuf, TRUE);
+
+	return IMAP_SUCCESS;
+}
+
 static gint imap_ok(gint sock, GPtrArray *argbuf)
 {
 	gint ok;
@@ -890,7 +1189,9 @@ static gchar *search_array_contain_str(GPtrArray *array, gchar *str)
 	gint i;
 
 	for (i = 0; i < array->len; i++) {
-		gchar *tmp = g_ptr_array_index(array, i);
+		gchar *tmp;
+
+		tmp = g_ptr_array_index(array, i);
 		if (strstr(tmp, str) != NULL)
 			return tmp;
 	}
@@ -932,3 +1233,8 @@ static gint imap_gen_recv(gint sock, gchar *buf, gint size)
 
 	return IMAP_SUCCESS;
 }
+
+static void imap_path_subst_slash_to_dot(gchar *str)
+{	
+	subst_char(str, '/', '.');
+}
