@@ -1,6 +1,30 @@
-/* import.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#include "defs.h"
 
 #include <glib.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkwindow.h>
@@ -13,12 +37,16 @@
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtksignal.h>
 
+#include "intl.h"
 #include "main.h"
 #include "inc.h"
 #include "mbox.h"
+#include "folderview.h"
 #include "filesel.h"
 #include "foldersel.h"
 #include "gtkutils.h"
+#include "manage_window.h"
+#include "folder.h"
 
 static GtkWidget *window;
 static GtkWidget *file_entry;
@@ -34,6 +62,7 @@ static void import_ok_cb(GtkWidget *widget, gpointer data);
 static void import_cancel_cb(GtkWidget *widget, gpointer data);
 static void import_filesel_cb(GtkWidget *widget, gpointer data);
 static void import_destsel_cb(GtkWidget *widget, gpointer data);
+static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
 
 gint import_mbox(void)
 {
@@ -41,23 +70,32 @@ gint import_mbox(void)
 
 	if (!window)
 		import_create();
-	gtk_widget_show(window);
+	else
+		gtk_widget_show(window);
+
+	manage_window_set_transient(GTK_WINDOW(window));
 
 	gtk_main();
 
 	if (import_ack) {
 		gchar *filename, *destdir;
+		FolderItem *dest;
 
 		filename = gtk_entry_get_text(GTK_ENTRY(file_entry));
 		destdir = gtk_entry_get_text(GTK_ENTRY(dest_entry));
 		if (filename && *filename) {
 			if (!destdir || !*destdir) {
-				destdir = g_strconcat
-					(maildir, "/", INBOX_DIR, NULL);
-				ok = proc_mbox(destdir, filename);
-				g_free(destdir);
+				dest = folder_find_item_from_path(INBOX_DIR);
 			} else
-				ok = proc_mbox(destdir, filename);
+				dest = folder_find_item_from_path(destdir);
+
+			if (!dest) {
+				g_warning("Can't find the folder.\n");
+			} else {
+				ok = proc_mbox(dest, filename, NULL);
+				folder_item_scan(dest);
+				folderview_update_item(dest, TRUE);
+			}
 		}
 	}
 
@@ -83,6 +121,12 @@ static void import_create(void)
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
 	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
 			   GTK_SIGNAL_FUNC(import_cancel_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
+			   GTK_SIGNAL_FUNC(key_pressed), NULL);
+	gtk_signal_connect(GTK_OBJECT(window), "focus_in_event",
+			   GTK_SIGNAL_FUNC(manage_window_focus_in), NULL);
+	gtk_signal_connect(GTK_OBJECT(window), "focus_out_event",
+			   GTK_SIGNAL_FUNC(manage_window_focus_out), NULL);
 
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -154,20 +198,22 @@ static void import_filesel_cb(GtkWidget *widget, gpointer data)
 {
 	gchar *filename;
 
-	filename = filesel_select_file(_("Select importing file"));
+	filename = filesel_select_file(_("Select importing file"), NULL);
 	if (filename)
 		gtk_entry_set_text(GTK_ENTRY(file_entry), filename);
 }
 
 static void import_destsel_cb(GtkWidget *widget, gpointer data)
 {
-	gchar *destdir;
+	FolderItem *dest;
 
-	//destdir = filesel_select_file(_("Select destination directory"));
-	destdir = foldersel_folder_sel(NULL);
-	if (destdir) {
-		gtk_entry_set_text(GTK_ENTRY(dest_entry), maildir);
-		gtk_entry_append_text(GTK_ENTRY(dest_entry), "/");
-		gtk_entry_append_text(GTK_ENTRY(dest_entry), destdir);
-	}
+	dest = foldersel_folder_sel(NULL);
+	if (dest && dest->path)
+		gtk_entry_set_text(GTK_ENTRY(dest_entry), dest->path);
+}
+
+static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
+{
+	if (event && event->keyval == GDK_Escape)
+		import_cancel_cb(NULL, NULL);
 }
