@@ -1,4 +1,21 @@
-/* inputdialog.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -64,7 +81,9 @@ gchar *input_dialog(const gchar *title, const gchar *message,
 
 	gtk_main();
 
+	manage_window_focus_out(dialog, NULL, NULL);
 	gtk_widget_hide(dialog);
+	gtk_entry_set_visibility(GTK_ENTRY(entry), TRUE);
 
 	if (ack) {
 		str = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
@@ -73,13 +92,27 @@ gchar *input_dialog(const gchar *title, const gchar *message,
 	} else
 		str = NULL;
 
+	GTK_EVENTS_FLUSH();
+
 	debug_print("return string = %s\n", str ? str : "(none)");
 	return str;
 }
 
+gchar *input_dialog_with_invisible(const gchar *title, const gchar *message,
+				   const gchar *default_string)
+{
+	if (dialog && GTK_WIDGET_VISIBLE(dialog)) return NULL;
+
+	if (!dialog)
+		input_dialog_create();
+
+	gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
+
+	return input_dialog(title, message, default_string);
+}
+
 static void input_dialog_create(void)
 {
-	//GtkWidget *label;
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *confirm_area;
@@ -95,8 +128,10 @@ static void input_dialog_create(void)
 			   GTK_SIGNAL_FUNC(cancel_clicked), NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event",
 			   GTK_SIGNAL_FUNC(key_pressed), NULL);
-	//gtk_signal_connect(GTK_OBJECT(dialog), "focus_in_event",
-	//		   GTK_SIGNAL_FUNC(manage_window_focus_in), NULL);
+	gtk_signal_connect(GTK_OBJECT(dialog), "focus_in_event",
+			   GTK_SIGNAL_FUNC(manage_window_focus_in), NULL);
+	gtk_signal_connect(GTK_OBJECT(dialog), "focus_out_event",
+			   GTK_SIGNAL_FUNC(manage_window_focus_out), NULL);
 
 	gtk_widget_realize(dialog);
 
@@ -109,6 +144,7 @@ static void input_dialog_create(void)
 
 	msg_label = gtk_label_new("");
 	gtk_box_pack_start(GTK_BOX(hbox), msg_label, FALSE, FALSE, 0);
+	gtk_label_set_justify(GTK_LABEL(msg_label), GTK_JUSTIFY_LEFT);
 
 	entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
