@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +67,18 @@
 #include "setup.h"
 #include "utils.h"
 
+gchar *prog_version;
+gboolean debug_mode = FALSE;
+
+static gint lock_socket = -1;
+static gint lock_socket_tag = 0;
+
+static struct Cmd {
+	gboolean receive;
+	gboolean receive_all;
+	gboolean compose;
+	const gchar *compose_mailto;
+} cmd;
 
 static void parse_cmd_opt(int argc, char *argv[]);
 
@@ -74,8 +86,13 @@ static void parse_cmd_opt(int argc, char *argv[]);
 static void idle_function_for_gpgme(void);
 #endif /* USE_GPGME */
 
-gchar *prog_version;
-gboolean debug_mode = FALSE;
+static gint prohibit_duplicate_launch	(void);
+static void lock_socket_input_cb	(gpointer	   data,
+					 gint		   source,
+					 GdkInputCondition condition);
+static gchar *get_socket_name		(void);
+
+static void open_compose_new_with_recipient	(const gchar	*address);
 
 #if 0
 /* for gettext */
@@ -152,6 +169,10 @@ int main(int argc, char *argv[])
 
 	CHDIR_RETURN_VAL_IF_FAIL(g_get_home_dir(), 1);
 
+	/* check and create unix domain socket */
+	lock_socket = prohibit_duplicate_launch();
+	if (lock_socket < 0) return 0;
+    
 	/* backup if old rc file exists */
 	if (is_file_exist(RC_DIR)) {
 		if (rename(RC_DIR, RC_DIR ".bak") < 0)
@@ -183,6 +204,12 @@ int main(int argc, char *argv[])
 		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
 	folderview = mainwin->folderview;
 
+	/* register the callback of unix domain socket input */
+	lock_socket_tag = gdk_input_add(lock_socket,
+					GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
+					lock_socket_input_cb,
+					mainwin);
+
 	account_read_config_all();
 	account_save_config_all();
 
@@ -190,11 +217,15 @@ int main(int argc, char *argv[])
 	account_set_missing_folder();
 	folderview_set(folderview);
 
-	if (prefs_common.chk_on_startup) {
+	if (cmd.receive_all)
+		inc_all_account_mail(mainwin);
+	else if (prefs_common.chk_on_startup || cmd.receive)
 		inc_mail(mainwin);
-	} else {
+	else
 		gtk_widget_grab_focus(folderview->ctree);
-	}
+
+	if (cmd.compose)
+		open_compose_new_with_recipient(cmd.compose_mailto);
 
 	/* ignore SIGPIPE signal for preventing sudden death of program */
 	signal(SIGPIPE, SIG_IGN);
@@ -211,8 +242,31 @@ static void parse_cmd_opt(int argc, char *argv[])
 	for (i = 1; i < argc; i++) {
 		if (!strncmp(argv[i], "--debug", 7))
 			debug_mode = TRUE;
-		else if (!strncmp(argv[i], "--help", 6)) {
-			g_print(_("Usage: sylpheed [--debug]\n"));
+		else if (!strncmp(argv[i], "--receive-all", 13))
+			cmd.receive_all = TRUE;
+		else if (!strncmp(argv[i], "--receive", 9))
+			cmd.receive = TRUE;
+		else if (!strncmp(argv[i], "--compose", 9)) {
+			const gchar *p = argv[i + 1];
+
+			cmd.compose = TRUE;
+			cmd.compose_mailto = NULL;
+			if (p && *p != '\0' && *p != '-') {
+				if (!strncmp(p, "mailto:", 7))
+					cmd.compose_mailto = p + 7;
+				else
+					cmd.compose_mailto = p;
+				i++;
+			}
+		} else if (!strncmp(argv[i], "--help", 6)) {
+			g_print(_("Usage: %s [OPTION]...\n"), g_get_prgname());
+
+			puts(_("  --compose [address]    open composition window"));
+			puts(_("  --receive              receive new messages"));
+			puts(_("  --receive-all          receive new messages of all accounts"));
+			puts(_("  --debug                debug mode"));
+			puts(_("  --help                 display this help and exit"));
+
 			exit(1);
 		}
 	}
@@ -250,6 +304,12 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	/* delete temporary files */
 	remove_all_files(get_mime_tmp_dir());
 
+	/* delete unix domain socket */
+	gdk_input_remove(lock_socket_tag);
+	sock_close(lock_socket);
+	filename = get_socket_name();
+	unlink(filename);
+
 	gtk_main_quit();
 }
 
@@ -260,3 +320,93 @@ static void idle_function_for_gpgme(void)
 		gtk_main_iteration();
 }
 #endif /* USE_GPGME */
+
+static gchar *get_socket_name(void)
+{
+	static gchar *filename = NULL;
+
+	if (filename == NULL) {
+		filename = g_strdup_printf("%s%csylpheed-%d",
+					   g_get_tmp_dir(), G_DIR_SEPARATOR,
+					   getuid());
+	}
+
+	return filename;
+}
+
+static gint prohibit_duplicate_launch(void)
+{
+	gint uxsock;
+	gchar *path;
+
+	path = get_socket_name();
+	uxsock = sock_connect_unix(path);
+	if (uxsock < 0) {
+		unlink(path);
+		return sock_open_unix(path);
+	}
+
+	/* remote command mode */
+
+	debug_print(_("another Sylpheed is already running.\n"));
+
+	if (cmd.receive_all)
+		sock_write(uxsock, "receive_all\n", 12);
+	else if (cmd.receive)
+		sock_write(uxsock, "receive\n", 8);
+	else if (cmd.compose) {
+		gchar *compose_str;
+
+		if (cmd.compose_mailto)
+			compose_str = g_strdup_printf("compose %s\n", cmd.compose_mailto);
+		else
+			compose_str = g_strdup("compose\n");
+
+		sock_write(uxsock, compose_str, strlen(compose_str));
+		g_free(compose_str);
+	} else
+		sock_write(uxsock, "popup\n", 6);
+
+	sock_close(uxsock);
+	return -1;
+}
+
+static void lock_socket_input_cb(gpointer data,
+				 gint source,
+				 GdkInputCondition condition)
+{
+	MainWindow *mainwin = (MainWindow *)data;
+	gint sock;
+	gchar buf[BUFFSIZE];
+
+	sock = sock_accept(source);
+	sock_read(sock, buf, sizeof(buf));
+	sock_close(sock);
+
+	if (!strncmp(buf, "popup", 5)){
+		main_window_popup(mainwin);
+	} else if (!strncmp(buf, "receive_all", 11)){
+		main_window_popup(mainwin);
+		inc_all_account_mail(mainwin);
+	} else if (!strncmp(buf, "receive", 7)){
+		main_window_popup(mainwin);
+		inc_mail(mainwin);
+	} else if (!strncmp(buf, "compose", 7)) {
+		open_compose_new_with_recipient(buf + strlen("compose") + 1);
+	}
+}
+
+static void open_compose_new_with_recipient(const gchar *address)
+{
+	gchar *addr = NULL;
+
+	if (address) {
+		Xstrdup_a(addr, address, return);
+		g_strstrip(addr);
+	}
+
+	if (addr && *addr != '\0')
+		compose_new_with_recipient(NULL, addr);
+	else
+		compose_new(NULL);
+}
