@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -203,6 +203,11 @@ static void reply_cb			(MainWindow	*mainwin,
 static void open_msg_cb			(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
+
+static void view_source_cb		(MainWindow	*mainwin,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void reedit_cb			(MainWindow	*mainwin,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -340,9 +345,6 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/Forward as an a_ttachment"),
 						"<shift><control>F", reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/Open in new _window"),	"<shift><control>N", open_msg_cb, 0, NULL},
-	{N_("/_Message/Re_edit"),		NULL, reedit_cb, 0, NULL},
-	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/M_ove..."),		"<alt>O", move_to_cb, 0, NULL},
 	{N_("/_Message/_Delete"),		"<alt>D", delete_cb,  0, NULL},
 	{N_("/_Message/_Mark"),			NULL, NULL, 0, "<Branch>"},
@@ -353,7 +355,11 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/_Mark/Mark it as _being read"),
 						NULL, mark_as_read_cb, 0, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Message/Open in new _window"),	"<shift><control>N", open_msg_cb, 0, NULL},
+	{N_("/_Message/View _source"),		"<control>U", view_source_cb, 0, NULL},
 	{N_("/_Message/Show all _header"),	"<control>H",	header_window_show_cb,	0, NULL},
+	{N_("/_Message/Re_edit"),		NULL, reedit_cb, 0, NULL},
+	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/_Code set"),		NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/_Code set/_Auto detect"),
 	 NULL, set_charset_cb, C_AUTO, "<RadioItem>"},
@@ -886,7 +892,7 @@ void main_window_get_position(MainWindow *mainwin)
 {
 	gint x, y;
 
-	gdk_window_get_root_origin(mainwin->window->window, &x, &y);
+	gtkut_widget_get_uposition(mainwin->window, &x, &y);
 
 	prefs_common.mainview_x = x;
 	prefs_common.mainview_y = y;
@@ -1000,6 +1006,8 @@ void main_window_set_menu_sensitive(MainWindow *mainwin, gint selection)
 	menu_set_sensitive(ifactory, "/Message/Forward", sens);
 	menu_set_sensitive(ifactory, "/Message/Forward as an attachment", sens);
 	menu_set_sensitive(ifactory, "/Message/Open in new window", sens);
+	menu_set_sensitive(ifactory, "/Message/Show all header", sens);
+	menu_set_sensitive(ifactory, "/Message/View source", sens);
 	if (sens && (!mainwin->summaryview->folder_item ||
 		     mainwin->summaryview->folder_item->stype != F_DRAFT))
 		sens = FALSE;
@@ -1031,6 +1039,57 @@ void main_window_set_menu_sensitive(MainWindow *mainwin, gint selection)
 	menu_set_sensitive(ifactory, "/Summary/Sort", sens);
 }
 
+void main_window_popup(MainWindow *mainwin)
+{
+	gint x, y;
+	gint sx, sy;
+	GtkWidget *widget;
+
+	gdk_window_get_origin(mainwin->window->window, &x, &y);
+	sx = gdk_screen_width();
+	sy = gdk_screen_height();
+	x %= sx; if (x < 0) x += sx;
+	y %= sy; if (y < 0) y += sy;
+	gdk_window_move(mainwin->window->window, x, y);
+	gdk_window_raise(mainwin->window->window);
+
+	debug_print("window position: x = %d, y = %d\n", x, y);
+
+	switch (mainwin->type) {
+	case SEPARATE_FOLDER:
+		widget = mainwin->win.sep_folder.folderwin;
+		gdk_window_get_origin(widget->window, &x, &y);
+		x %= sx; if (x < 0) x += sx;
+		y %= sy; if (y < 0) y += sy;
+		gdk_window_move(widget->window, x, y);
+		gdk_window_raise(widget->window);
+		break;
+	case SEPARATE_MESSAGE:
+		widget = mainwin->win.sep_message.messagewin;
+		gdk_window_get_origin(widget->window, &x, &y);
+		x %= sx; if (x < 0) x += sx;
+		y %= sy; if (y < 0) y += sy;
+		gdk_window_move(widget->window, x, y);
+		gdk_window_raise(widget->window);
+		break;
+	case SEPARATE_BOTH:
+		widget = mainwin->win.sep_both.folderwin;
+		gdk_window_get_origin(widget->window, &x, &y);
+		x %= sx; if (x < 0) x += sx;
+		y %= sy; if (y < 0) y += sy;
+		gdk_window_move(widget->window, x, y);
+		gdk_window_raise(widget->window);
+		widget = mainwin->win.sep_both.messagewin;
+		gdk_window_get_origin(widget->window, &x, &y);
+		x %= sx; if (x < 0) x += sx;
+		y %= sy; if (y < 0) y += sy;
+		gdk_window_move(widget->window, x, y);
+		gdk_window_raise(widget->window);
+		break;
+	default:
+	}
+}
+
 static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 {
 	GtkWidget *folderwin = NULL;
@@ -1735,6 +1794,12 @@ static void open_msg_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	summary_open_msg(mainwin->summaryview);
 }
 
+static void view_source_cb(MainWindow *mainwin, guint action,
+			   GtkWidget *widget)
+{
+	summary_view_source(mainwin->summaryview);
+}
+
 static void reedit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	summary_reedit(mainwin->summaryview);
