@@ -26,8 +26,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <errno.h>
 
 #include "intl.h"
@@ -272,7 +270,7 @@ PrefFile *prefs_write_open(const gchar *path)
 		return NULL;
 	}
 
-	if (chmod(tmppath, S_IRUSR|S_IWUSR) < 0)
+	if (change_file_mode_rw(fp, tmppath) < 0)
 		FILE_OP_ERROR(tmppath, "chmod");
 
 	g_free(tmppath);
