@@ -94,6 +94,9 @@ static struct Send {
 	GtkWidget *autobcc_entry;
 	GtkWidget *autoreplyto_chkbtn;
 	GtkWidget *autoreplyto_entry;
+
+	GtkWidget *smtp_auth_chkbtn;
+	GtkWidget *pop_bfr_smtp_chkbtn;
 } send;
 
 static struct Compose {
@@ -145,9 +148,6 @@ static PrefParam param[] = {
 	{"nntp_server", NULL, &tmp_ac_prefs.nntp_server, P_STRING,
 	 &basic.nntpserv_entry, prefs_set_data_from_entry, prefs_set_entry},
 
-	{"use_smtp_auth", "FALSE", &tmp_ac_prefs.use_smtp_auth, P_BOOL,
-	 NULL, NULL, NULL},
-
 	{"use_nntp_auth", "FALSE", &tmp_ac_prefs.use_nntp_auth, P_BOOL,
 	 NULL, NULL, NULL},
 
@@ -214,6 +214,14 @@ static PrefParam param[] = {
 	 &send.autoreplyto_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
 
+	{"use_smtp_auth", "FALSE", &tmp_ac_prefs.use_smtp_auth, P_BOOL,
+	 &send.smtp_auth_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
+	{"pop_before_smtp", "FALSE", &tmp_ac_prefs.pop_before_smtp, P_BOOL,
+	 &send.pop_bfr_smtp_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
 	/* Compose */
 	{"signature_path", "~/"DEFAULT_SIGNATURE, &tmp_ac_prefs.sig_path, P_STRING,
 	 &compose.sigpath_entry,
@@ -262,7 +270,7 @@ static gint prefs_account_apply			(void);
 static void prefs_account_cancel		(void);
 
 #define VSPACING	12
-#define VSPACING_NARROW	6
+#define VSPACING_NARROW	4
 #define BOX_BORDER	16
 
 void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
@@ -762,6 +770,10 @@ static void prefs_account_send_create(void)
 	GtkWidget *autobcc_entry;
 	GtkWidget *autoreplyto_chkbtn;
 	GtkWidget *autoreplyto_entry;
+	GtkWidget *frame3;
+	GtkWidget *vbox3;
+	GtkWidget *smtp_auth_chkbtn;
+	GtkWidget *pop_bfr_smtp_chkbtn;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -842,6 +854,19 @@ static void prefs_account_send_create(void)
 
 	SET_TOGGLE_SENSITIVITY (autoreplyto_chkbtn, autoreplyto_entry);
 
+	PACK_FRAME (vbox1, frame3, _("Authentication"));
+
+	vbox3 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox3);
+	gtk_container_add (GTK_CONTAINER (frame3), vbox3);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox3), 8);
+
+	PACK_CHECK_BUTTON (vbox3, smtp_auth_chkbtn,
+		_("SMTP Authentication (SMTP AUTH)"));
+	PACK_CHECK_BUTTON (vbox3, pop_bfr_smtp_chkbtn,
+		_("Authenticate with POP3 before sending"));
+	gtk_widget_set_sensitive(pop_bfr_smtp_chkbtn, FALSE);
+
 	send.date_chkbtn      = date_chkbtn;
 	send.msgid_chkbtn     = msgid_chkbtn;
 	send.customhdr_chkbtn = customhdr_chkbtn;
@@ -852,6 +877,9 @@ static void prefs_account_send_create(void)
 	send.autobcc_entry      = autobcc_entry;
 	send.autoreplyto_chkbtn = autoreplyto_chkbtn;
 	send.autoreplyto_entry  = autoreplyto_entry;
+
+	send.smtp_auth_chkbtn    = smtp_auth_chkbtn;
+	send.pop_bfr_smtp_chkbtn = pop_bfr_smtp_chkbtn;
 }
 
 static void prefs_account_compose_create(void)
