@@ -313,9 +313,9 @@ static PrefParam param[] = {
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
 	/* MIME viewer */
-	{"mime_image_viewer", "display %s",
+	{"mime_image_viewer", "display '%s'",
 	 &prefs_common.mime_image_viewer, P_STRING, NULL, NULL, NULL},
-	{"mime_audio_player", "play %s",
+	{"mime_audio_player", "play '%s'",
 	 &prefs_common.mime_audio_player, P_STRING, NULL, NULL, NULL},
 
 	{"show_mark", "TRUE", &prefs_common.show_mark, P_BOOL,
@@ -446,7 +446,7 @@ static PrefParam param[] = {
 };
 
 #define VSPACING	12
-#define VSPACING_NARROW	6
+#define VSPACING_NARROW	4
 #define VBOX_BORDER	16
 #define PREFSBUFSIZE	1024
 
