@@ -498,7 +498,7 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 	g_return_val_if_fail(mimeinfo != NULL, NULL);
 
 	if (!outfp) {
-		outfp = tmpfile();
+		outfp = my_tmpfile();
 		if (!outfp) {
 			perror("tmpfile");
 			return NULL;
@@ -658,7 +658,7 @@ FILE *procmime_get_text_part(MsgInfo *msginfo)
 		return NULL;
 	}
 
-	if ((outfp = tmpfile()) == NULL) {
+	if ((outfp = my_tmpfile()) == NULL) {
 		perror("tmpfile");
 		fclose(tmpfp);
 		procmime_mimeinfo_free_all(mimeinfo);
