@@ -582,9 +582,12 @@ gboolean procmsg_msg_exist(MsgInfo *msginfo)
 void procmsg_clean_trash(void)
 {
 	FolderItem *trash;
+	GList *cur;
 
-	trash = folder_get_default_trash();
-	if (trash) folder_item_remove_all_msg(trash);
+	for (cur = folder_get_list(); cur != NULL; cur = cur->next) {
+		trash = FOLDER(cur->data)->trash;
+		if (trash) folder_item_remove_all_msg(trash);
+	}
 }
 
 gint procmsg_send_queue(void)
