@@ -42,9 +42,11 @@
 
 static gint send_message_smtp	(GSList *to_list, const gchar *from,
 				 const gchar *server, gushort port,
-				 const gchar *domain, FILE *fp);
+				 const gchar *domain, const gchar *userid,
+				 const gchar *passwd, gboolean use_smtp_auth,
+				 FILE *fp);
 static gint send_smtp_open	(const gchar *server, gushort port,
-				 const gchar *domain);
+				 const gchar *domain, gboolean use_smtp_auth);
 
 #define SEND_EXIT_IF_ERROR(f, s) \
 { \
@@ -86,7 +88,9 @@ gint send_message(const gchar *file, PrefsAccount *ac_prefs, GSList *to_list)
 	domain = ac_prefs->set_domain ? ac_prefs->domain : NULL;
 
 	val = send_message_smtp(to_list, ac_prefs->address,
-				ac_prefs->smtp_server, port, domain, fp);
+				ac_prefs->smtp_server, port, domain,
+				ac_prefs->userid, ac_prefs->passwd,
+				ac_prefs->use_smtp_auth, fp);
 
 	fclose(fp);
 	return val;
@@ -126,10 +130,10 @@ gint send_message_queue(const gchar *file)
 
 		switch (hnum) {
 		case Q_SENDER:
-			from = g_strdup(p);
+			if (!from) from = g_strdup(p);
 			break;
 		case Q_SMTPSERVER:
-			server = g_strdup(p);
+			if (!server) server = g_strdup(p);
 			break;
 		case Q_RECIPIENTS:
 			to_list = address_list_append(to_list, p);
@@ -144,14 +148,26 @@ gint send_message_queue(const gchar *file)
 	} else {
 		gushort port;
 		gchar *domain;
+		PrefsAccount *ac;
 
-		/* TODO: the port number and domain should be in the
-		   queueing header */
-		port = cur_account && cur_account->set_smtpport ?
-			cur_account->smtpport : SMTP_PORT;
-		domain = cur_account->set_domain ? cur_account->domain : NULL;
-		val = send_message_smtp
-			(to_list, from, server, port, domain, fp);
+		ac = account_find_from_smtp_server(from, server);
+		if (!ac) {
+			g_warning(_("Account not found. Using current account...\n"));
+			ac = cur_account;
+		}
+
+		if (ac) {
+			port = ac->set_smtpport ? ac->smtpport : SMTP_PORT;
+			domain = ac->set_domain ? ac->domain : NULL;
+			val = send_message_smtp
+				(to_list, from, server, port, domain,
+				 ac->userid, ac->passwd, ac->use_smtp_auth, fp);
+		} else {
+			g_warning(_("Account not found.\n"));
+			val = send_message_smtp
+				(to_list, from, server, SMTP_PORT, NULL,
+				 NULL, NULL, FALSE, fp);
+		}
 	}
 
 	slist_free_strings(to_list);
@@ -165,7 +181,9 @@ gint send_message_queue(const gchar *file)
 
 static gint send_message_smtp(GSList *to_list, const gchar *from,
 			      const gchar *server, gushort port,
-			      const gchar *domain, FILE *fp)
+			      const gchar *domain, const gchar *userid,
+			      const gchar* passwd, gboolean use_smtp_auth,
+			      FILE *fp)
 {
 	gint smtp_sock;
 	gchar buf[BUFFSIZE];
@@ -178,9 +196,12 @@ static gint send_message_smtp(GSList *to_list, const gchar *from,
 
 	log_message(_("Connecting to SMTP server: %s ...\n"), server);
 
-	SEND_EXIT_IF_ERROR((smtp_sock = send_smtp_open(server, port, domain)),
+	SEND_EXIT_IF_ERROR((smtp_sock = send_smtp_open
+				(server, port, domain, use_smtp_auth)),
 			   "connecting to server");
-	SEND_EXIT_IF_NOTOK(smtp_from(smtp_sock, from), "sending MAIL FROM");
+	SEND_EXIT_IF_NOTOK
+		(smtp_from(smtp_sock, from, userid, passwd, use_smtp_auth),
+		 "sending MAIL FROM");
 	for (cur = to_list; cur != NULL; cur = cur->next)
 		SEND_EXIT_IF_NOTOK(smtp_rcpt(smtp_sock, (gchar *)cur->data),
 				   "sending RCPT TO");
@@ -206,7 +227,7 @@ static gint send_message_smtp(GSList *to_list, const gchar *from,
 }
 
 static gint send_smtp_open(const gchar *server, gushort port,
-			   const gchar *domain)
+			   const gchar *domain, gboolean use_smtp_auth)
 {
 	SockInfo *sockinfo;
 	gint sock;
@@ -223,7 +244,8 @@ static gint send_smtp_open(const gchar *server, gushort port,
 	sock_sockinfo_free(sockinfo);
 
 	if (smtp_ok(sock) == SM_OK) {
-		val = smtp_helo(sock, domain ? domain : get_domain_name());
+		val = smtp_helo(sock, domain ? domain : get_domain_name(),
+				use_smtp_auth);
 		if (val == SM_OK) return sock;
 	}
 
