@@ -20,6 +20,8 @@
 #ifndef __SMTP_H__
 #define __SMTP_H__
 
+#include <glib.h>
+
 #define	SMTPBUFSIZE		256
 
 #define	SM_OK			0
@@ -30,8 +32,9 @@
 #define	ESMTP_SIZE		0x02
 #define	ESMTP_ETRN		0x04
 
-gint smtp_helo(gint sock, const char *hostname);
-gint smtp_from(gint sock, const gchar *from);
+gint smtp_helo(gint sock, const char *hostname, gboolean use_smtp_auth);
+gint smtp_from(gint sock, const gchar *from, const gchar *userid,
+	       const gchar *passwd, gboolean use_smtp_auth);
 gint smtp_rcpt(gint sock, const gchar *to);
 gint smtp_data(gint sock);
 gint smtp_rset(gint sock);
