@@ -25,6 +25,7 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -52,6 +53,69 @@ static gint sock_connect_by_getaddrinfo	(const gchar	*hostname,
 					 gushort	 port);
 #endif
 
+gint sock_connect_unix(const gchar *path)
+{
+	gint sock;
+	struct sockaddr_un addr;
+
+	sock = socket(PF_UNIX, SOCK_STREAM, 0);
+	if (sock < 0) {
+		perror("sock_connect_unix(): socket");
+		return -1;
+	}
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sun_family = AF_UNIX;
+	strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
+
+	if (connect(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+		close(sock);
+		return -1;
+	}
+
+	return sock;
+}
+
+gint sock_open_unix(const gchar *path)
+{
+	gint sock;
+	struct sockaddr_un addr;
+
+	sock = socket(PF_UNIX, SOCK_STREAM, 0);
+
+	if (sock < 0) {
+		perror("sock_open_unix(): socket");
+		return -1;
+	}
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sun_family = AF_UNIX;
+	strncpy(addr.sun_path, path, sizeof(addr.sun_path) - 1);
+
+	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+		perror("bind");
+		close(sock);
+		return -1;
+	}
+
+	if (listen(sock, 1) < 0) {
+		perror("listen");
+		close(sock);
+		return -1;		
+	}
+
+	return sock;
+}
+
+gint sock_accept(gint sock)
+{
+	struct sockaddr_in caddr;
+	gint caddr_len;
+
+	caddr_len = sizeof(caddr);
+	return accept(sock, (struct sockaddr *)&caddr, &caddr_len);
+}
+
 gint sock_set_nonblocking_mode(gint sock, gboolean nonblock)
 {
 	gint flags;
@@ -247,7 +311,8 @@ SockInfo *sock_connect(const gchar *hostname, gushort port)
 static void sock_connect_thread(SockInfo *sockinfo)
 {
 #ifdef INET6
-	if ((sockinfo->sock = sock_connect_by_getaddrinfo(hostname, port)) < 0)
+	if ((sockinfo->sock = sock_connect_by_getaddrinfo
+		(sockinfo->hostname, sockinfo->port)) < 0)
 		pthread_exit((void *)1);
 #else
 	if (sock_connect_by_hostname(sockinfo->sock, sockinfo->hostname,
