@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,8 +45,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
-#include <dirent.h>
-#include <sys/stat.h>
 
 #include "intl.h"
 #include "main.h"
@@ -59,6 +57,7 @@
 #include "procmsg.h"
 #include "procheader.h"
 #include "headerwindow.h"
+#include "sourcewindow.h"
 #include "prefs_common.h"
 #include "account.h"
 #include "compose.h"
@@ -191,8 +190,8 @@ static void summary_button_released	(GtkWidget		*ctree,
 static void summary_key_pressed		(GtkWidget		*ctree,
 					 GdkEventKey		*event,
 					 SummaryView		*summaryview);
-static void summary_open_row            (GtkSCTree      *sctree,
-					 SummaryView    *summaryview);
+static void summary_open_row		(GtkSCTree		*sctree,
+					 SummaryView		*summaryview);
 static void summary_selected		(GtkCTree		*ctree,
 					 GtkCTreeNode		*row,
 					 gint			 column,
@@ -267,6 +266,7 @@ static GtkItemFactoryEntry summary_popup_entries[] =
 					NULL, summary_reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/---"),			NULL, NULL,		0, "<Separator>"},
 	{N_("/Open in new _window"),	NULL, summary_open_msg,	0, NULL},
+	{N_("/View so_urce"),		NULL, summary_view_source, 0, NULL},
 	{N_("/Show all _header"),	NULL, summary_show_all_header_cb, 0, NULL},
 	{N_("/Re_edit"),		NULL, summary_reedit,   0, NULL},
 	{N_("/---"),			NULL, NULL,		0, "<Separator>"},
@@ -465,7 +465,6 @@ SummaryView *summary_create(void)
 			   GTK_SIGNAL_FUNC(summary_col_resized), summaryview);
         gtk_signal_connect(GTK_OBJECT(ctree), "open_row",
 			   GTK_SIGNAL_FUNC(summary_open_row), summaryview);
-	
 	gtk_signal_connect(GTK_OBJECT(toggle_eventbox), "button_press_event",
 			   GTK_SIGNAL_FUNC(summary_toggle_pressed),
 			   summaryview);
@@ -613,7 +612,7 @@ gboolean summary_show(SummaryView *summaryview, FolderItem *item,
 			   GTK_SIGNAL_FUNC(summary_col_resized), summaryview);
         gtk_signal_connect(GTK_OBJECT(ctree), "open_row",
 			   GTK_SIGNAL_FUNC(summary_open_row), summaryview);
-	
+
 	/*connect drag and drop signal*/
 	gtk_signal_connect (GTK_OBJECT (ctree),"start_drag",
 			    GTK_SIGNAL_FUNC (summary_start_drag),
@@ -741,6 +740,7 @@ static void summary_set_menu_sensitive(SummaryView *summaryview)
 	menu_set_sensitive(ifactory, "/Forward as an attachment", sens);
 
 	menu_set_sensitive(ifactory, "/Open in new window", sens);
+	menu_set_sensitive(ifactory, "/View source", sens);
 	menu_set_sensitive(ifactory, "/Show all header", sens);
 	if (summaryview->folder_item->stype == F_DRAFT)
 		menu_set_sensitive(ifactory, "/Reedit", sens);
@@ -1203,7 +1203,7 @@ gint summary_write_cache(SummaryView *summaryview)
 		g_free(cachefile);
 		return -1;
 	}
-	if (chmod(cachefile, S_IRUSR|S_IWUSR) < 0)
+	if (change_file_mode_rw(fps.cache_fp, cachefile) < 0)
 		FILE_OP_ERROR(cachefile, "chmod");
 	g_free(cachefile);
 
@@ -1214,7 +1214,7 @@ gint summary_write_cache(SummaryView *summaryview)
 		g_free(markfile);
 		return -1;
 	}
-	if (chmod(markfile, S_IRUSR|S_IWUSR) < 0)
+	if (change_file_mode_rw(fps.mark_fp, markfile) < 0)
 		FILE_OP_ERROR(markfile, "chmod");
 	g_free(markfile);
 
@@ -1368,6 +1368,20 @@ void summary_open_msg(SummaryView *summaryview)
 	summary_display_msg(summaryview, summaryview->selected, TRUE);
 }
 
+void summary_view_source(SummaryView * summaryview)
+{
+	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
+	MsgInfo *msginfo;
+	SourceWindow *srcwin;
+
+	if (!summaryview->selected) return;
+
+	srcwin = source_window_create();
+	msginfo = gtk_ctree_node_get_row_data(ctree, summaryview->selected);
+	source_window_show_msg(srcwin, msginfo);
+	source_window_show(srcwin);
+}
+
 void summary_reedit(SummaryView *summaryview)
 {
 	MsgInfo *msginfo;
@@ -2144,7 +2158,8 @@ static void summary_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	} else if (event->button == 2) {
 		summaryview->display_msg = TRUE;
 	} else if (event->button == 1) {
-		if (!prefs_common.emulate_emacs)
+		if (!prefs_common.emulate_emacs &&
+		    summaryview->msg_is_toggled_on)
 			summaryview->display_msg = TRUE;
 	}
 }
@@ -2323,12 +2338,14 @@ static void summary_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	}
 }
 
-static void summary_open_row(GtkSCTree * sctree, SummaryView *summaryview)
+static void summary_open_row(GtkSCTree *sctree, SummaryView *summaryview)
 {
 	if (summaryview->folder_item->stype == F_DRAFT)
 		summary_reedit(summaryview);
 	else
 		summary_open_msg(summaryview);
+
+	summaryview->display_msg = FALSE;
 }
 
 static void summary_selected(GtkCTree *ctree, GtkCTreeNode *row,
@@ -2345,7 +2362,7 @@ static void summary_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 	summaryview->selected = row;
 
-	if (summaryview->msg_is_toggled_on && summaryview->display_msg)
+	if (summaryview->display_msg)
 		summary_display_msg(summaryview, row, FALSE);
 
 	summaryview->display_msg = FALSE;
@@ -2468,25 +2485,26 @@ void summary_change_display_item(SummaryView *summaryview)
 	gtk_clist_set_column_visibility(clist, S_COL_SUBJECT, prefs_common.show_subject);
 }
 
-static void summary_start_drag (GtkWidget *widget,
-				int button,
-				GdkEvent *event,
-				SummaryView      *summaryview)
+static void summary_start_drag (GtkWidget   *widget,
+				gint         button,
+				GdkEvent    *event,
+				SummaryView *summaryview)
 {
 	GtkTargetList *list;
 	GdkDragContext *context;
 
-	g_return_if_fail(summaryview);
-	g_return_if_fail(summaryview->folder_item);
-	g_return_if_fail(summaryview->folder_item->folder);
-	if ((summaryview->folder_item->folder->type != F_MH) ||
-	    (summaryview->selected == NULL))return ;
+	g_return_if_fail(summaryview != NULL);
+	g_return_if_fail(summaryview->folder_item != NULL);
+	g_return_if_fail(summaryview->folder_item->folder != NULL);
+	if (summaryview->folder_item->folder->type == F_NEWS ||
+	    summaryview->selected == NULL)
+		return;
 
-	list = gtk_target_list_new (summary_drag_types, 1);
-	
-	context = gtk_drag_begin (widget, list,
-				  GDK_ACTION_MOVE,button, event);
-	gtk_drag_set_icon_default (context);
+	list = gtk_target_list_new(summary_drag_types, 1);
+
+	context = gtk_drag_begin(widget, list,
+				 GDK_ACTION_MOVE, button, event);
+	gtk_drag_set_icon_default(context);
 }
 
 static void summary_drag_data_get(GtkWidget        *widget,
@@ -2496,34 +2514,39 @@ static void summary_drag_data_get(GtkWidget        *widget,
 				  guint             time,
 				  SummaryView      *summaryview)
 {
-	if(info == TARGET_MAIL_URI_LIST){
+	if (info == TARGET_MAIL_URI_LIST) {
 		GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
 		GList *cur;
 		MsgInfo *msginfo;
-		gchar *mail_list=NULL,*tmp1,*tmp2;
-		
-		for (cur = GTK_CLIST(ctree)->selection; cur != NULL; cur = cur->next){
-			msginfo = gtk_ctree_node_get_row_data( ctree,
-							      (GtkCTreeNode *)cur->data);
+		gchar *mail_list = NULL, *tmp1, *tmp2;
+
+		for (cur = GTK_CLIST(ctree)->selection;
+		     cur != NULL; cur = cur->next) {
+			msginfo = gtk_ctree_node_get_row_data
+				(ctree, GTK_CTREE_NODE(cur->data));
 			tmp2 = procmsg_get_message_file_path(msginfo);
+			if (!tmp2) continue;
 			tmp1 = g_strconcat("file:/", tmp2, NULL);
-			if(mail_list == NULL){
+			g_free(tmp2);
+
+			if (!mail_list) {
 				mail_list = tmp1;
-			}else{
-				tmp2 = g_strconcat(mail_list,tmp1,NULL);
+			} else {
+				tmp2 = g_strconcat(mail_list, tmp1, NULL);
 				g_free(mail_list);
 				g_free(tmp1);
 				mail_list = tmp2;
 			}
 		}
-		if(mail_list!=NULL){
+
+		if (mail_list != NULL) {
 			gtk_selection_data_set(selection_data,
 					       selection_data->target, 8,
 					       mail_list, strlen(mail_list));
 			g_free(mail_list);
 		} 
-	}else if(info == TARGET_DUMMY){
-		if(GTK_CLIST(summaryview->ctree)->selection)
+	} else if (info == TARGET_DUMMY) {
+		if (GTK_CLIST(summaryview->ctree)->selection)
 			gtk_selection_data_set(selection_data,
 					       selection_data->target, 8,
 					       "Dummy", 6);
