@@ -35,10 +35,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#if HAVE_LIBJCONV
-#  include <jconv.h>
-#endif
-
 #include "intl.h"
 #include "main.h"
 #include "summaryview.h"
@@ -48,6 +44,8 @@
 #include "utils.h"
 #include "gtkutils.h"
 #include "procmime.h"
+#include "html.h"
+#include "compose.h"
 
 #define FONT_LOAD(font, s) \
 { \
@@ -89,26 +87,21 @@ static GdkColor emphasis_color = {
 	(gushort)0xcfff
 };
 
-#if HAVE_LIBJCONV
 static GdkColor error_color = {
 	(gulong)0,
 	(gushort)0xefff,
 	(gushort)0,
 	(gushort)0
 };
-#endif
 
 static GdkFont *spacingfont;
 
-#if HAVE_LIBJCONV
+static void textview_show_html		(TextView	*textview,
+					 FILE		*fp,
+					 CodeConverter	*conv);
 static void textview_write_line		(TextView	*textview,
 					 const gchar	*str,
-					 const gchar	*charset);
-#else
-static void textview_write_line		(TextView	*textview,
-					 const gchar	*str,
-					 CodeConvFunc	 code_conv);
-#endif
+					 CodeConverter	*conv);
 static GPtrArray *textview_scan_header	(TextView	*textview,
 					 FILE		*fp);
 static void textview_show_header	(TextView	*textview,
@@ -256,9 +249,7 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 	const gchar *charset = NULL;
 	FILE *tmpfp;
 	GPtrArray *headers = NULL;
-#if !HAVE_LIBJCONV
-	CodeConvFunc code_conv;
-#endif
+	CodeConverter *conv;
 
 	g_return_if_fail(mimeinfo != NULL);
 	g_return_if_fail(fp != NULL);
@@ -319,9 +310,7 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 		charset = mimeinfo->charset;
 	textview_set_font(textview, charset);
 
-#if !HAVE_LIBJCONV
-	code_conv = conv_get_code_conv_func(charset);
-#endif
+	conv = conv_code_converter_new(charset);
 
 	textview_clear(textview);
 	text = GTK_TEXT(textview->text);
@@ -342,15 +331,16 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 	tmpfp = procmime_decode_content(NULL, fp, mimeinfo);
 	if (tmpfp) {
-		while (fgets(buf, sizeof(buf), tmpfp) != NULL)
-#if HAVE_LIBJCONV
-			textview_write_line(textview, buf, charset);
-#else
-			textview_write_line(textview, buf, code_conv);
-#endif
+		if (mimeinfo->mime_type == MIME_TEXT_HTML)
+			textview_show_html(textview, tmpfp, conv);
+		else
+			while (fgets(buf, sizeof(buf), tmpfp) != NULL)
+				textview_write_line(textview, buf, conv);
 		fclose(tmpfp);
 	}
 
+	conv_code_converter_destroy(conv);
+
 	gtk_text_thaw(text);
 }
 
@@ -390,37 +380,271 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	gtk_text_thaw(text);
 }
 
-#if HAVE_LIBJCONV
+static void textview_show_html(TextView *textview, FILE *fp,
+			       CodeConverter *conv)
+{
+	HTMLParser *parser;
+	gchar *str;
+
+	parser = html_parser_new(fp, conv);
+	g_return_if_fail(parser != NULL);
+
+	while ((str = html_parse(parser)) != NULL) {
+		textview_write_line(textview, str, NULL);
+	}
+	html_parser_destroy(parser);
+}
+
+/* get_uri_part() - retrieves a URI starting from scanpos.
+		    Returns TRUE if succesful */
+static gboolean get_uri_part(const gchar *start, const gchar *scanpos,
+			     const gchar **bp, const gchar **ep)
+{
+	const gchar *ep_;
+
+	g_return_val_if_fail(start != NULL, FALSE);
+	g_return_val_if_fail(scanpos != NULL, FALSE);
+	g_return_val_if_fail(bp != NULL, FALSE);
+	g_return_val_if_fail(ep != NULL, FALSE);
+
+	*bp = scanpos;
+
+	/* find end point of URI */
+	for (ep_ = scanpos; *ep_ != '\0'; ep_++) {
+		if (!isgraph(*ep_) || !isascii(*ep_) || strchr("()<>\"", *ep_))
+			break;
+	}
+
+	/* no punctuation at end of string */
+
+	/* FIXME: this stripping of trailing punctuations may bite with other URIs.
+	 * should pass some URI type to this function and decide on that whether
+	 * to perform punctuation stripping */
+
+#define IS_REAL_PUNCT(ch) \
+	(ispunct(ch) && ((ch) != '/')) 
+
+	for (; ep_ - 1 > scanpos + 1 && IS_REAL_PUNCT(*(ep_ - 1)); ep_--)
+		;
+
+#undef IS_REAL_PUNCT
+
+	*ep = ep_;
+
+	return TRUE;		
+}
+
+static gchar *make_uri_string(const gchar *bp, const gchar *ep)
+{
+	return g_strndup(bp, ep - bp);
+}
+
+/* valid mail address characters */
+#define IS_RFC822_CHAR(ch) \
+	(isascii(ch) && \
+	 (ch) > 32   && \
+	 (ch) != 127 && \
+	 !isspace(ch) && \
+	 !strchr("()<>\"", (ch)))
+
+/* get_email_part() - retrieves an email address. Returns TRUE if succesful */
+static gboolean get_email_part(const gchar *start, const gchar *scanpos,
+			       const gchar **bp, const gchar **ep)
+{
+	/* more complex than the uri part because we need to scan back and forward starting from
+	 * the scan position. */
+	gboolean result = FALSE;
+	const gchar *bp_;
+	const gchar *ep_;
+
+	g_return_val_if_fail(start != NULL, FALSE);
+	g_return_val_if_fail(scanpos != NULL, FALSE);
+	g_return_val_if_fail(bp != NULL, FALSE);
+	g_return_val_if_fail(ep != NULL, FALSE);
+
+	/* scan start of address */
+	for (bp_ = scanpos - 1; bp_ >= start && IS_RFC822_CHAR(*bp_); bp_--)
+		;
+
+	/* TODO: should start with an alnum? */
+	bp_++;
+	for (; bp_ < scanpos && !isalnum(*bp_); bp_++)
+		;
+
+	if (bp_ != scanpos) {
+		/* scan end of address */
+		for (ep_ = scanpos + 1; *ep_ && IS_RFC822_CHAR(*ep_); ep_++)
+			;
+
+		/* TODO: really should terminate with an alnum? */
+		for (; ep_ > scanpos  && !isalnum(*ep_); --ep_)
+			;
+		ep_++;
+
+		if (ep_ > scanpos + 1) {
+			*ep = ep_;
+			*bp = bp_;
+			result = TRUE;
+		}
+	}
+
+	return result;
+}
+
+#undef IS_RFC822_CHAR
+
+static gchar *make_email_string(const gchar *bp, const gchar *ep)
+{
+	/* returns a mailto: URI; mailto: is also used to detect the
+	 * uri type later on in the button_pressed signal handler */
+	gchar *tmp;
+	gchar *result;
+
+	tmp = g_strndup(bp, ep - bp);
+	result = g_strconcat("mailto:", tmp, NULL);
+	g_free(tmp);
+
+	return result;
+}
+
+#define ADD_TXT_POS(bp_, ep_, pti_) \
+	if ((last->next = alloca(sizeof(struct txtpos))) != NULL) { \
+		last = last->next; \
+		last->bp = (bp_); last->ep = (ep_); last->pti = (pti_); \
+		last->next = NULL; \
+	} else { \
+		g_warning("alloc error scanning URIs\n"); \
+		gtk_text_insert(text, textview->msgfont, fg_color, NULL, \
+				linebuf, -1); \
+		return; \
+	}
+
+/* textview_make_clickable_parts() - colorizes clickable parts */
+static void textview_make_clickable_parts(TextView *textview, GtkText *text,
+					  GdkColor *fg_color,
+					  GdkColor *uri_color,
+					  const gchar *linebuf)
+{
+	/* parse table - in order of priority */
+	struct table {
+		const gchar *needle; /* token */
+
+		/* token search function */
+		gchar    *(*search)	(const gchar *haystack,
+					 const gchar *needle);
+		/* part parsing function */
+		gboolean  (*parse)	(const gchar *start,
+					 const gchar *scanpos,
+					 const gchar **bp_,
+					 const gchar **ep_);
+		/* part to URI function */
+		gchar    *(*build_uri)	(const gchar *bp,
+					 const gchar *ep);
+	};
+
+	static struct table parser[] = {
+		{"http://",  strcasestr, get_uri_part,   make_uri_string},
+		{"https://", strcasestr, get_uri_part,   make_uri_string},
+		{"ftp://",   strcasestr, get_uri_part,   make_uri_string},
+		{"mailto:",  strcasestr, get_uri_part,   make_uri_string},
+		{"@",        strcasestr, get_email_part, make_email_string}
+	};
+	const gint PARSE_ELEMS = sizeof parser / sizeof parser[0];
+
+	gint  n;
+	const gchar *walk, *bp, *ep;
+
+	struct txtpos {
+		const gchar	*bp, *ep;	/* text position */
+		gint		 pti;		/* index in parse table */
+		struct txtpos	*next;		/* next */
+	} head = {NULL, NULL, 0,  NULL}, *last = &head;
+
+	/* parse for clickable parts, and build a list of begin and end positions  */
+	for (walk = linebuf, n = 0;;) {
+		gint last_index = PARSE_ELEMS;
+		gchar *scanpos = NULL;
+
+		/* FIXME: this looks phony. scanning for anything in the parse table */
+		for (n = 0; n < PARSE_ELEMS; n++) {
+			gchar *tmp;
+
+			tmp = parser[n].search(walk, parser[n].needle);
+			if (tmp) {
+				if (scanpos == NULL || tmp < scanpos) {
+					scanpos = tmp;
+					last_index = n;
+				}
+			}					
+		}
+
+		if (scanpos) {
+			/* check if URI can be parsed */
+			if (parser[last_index].parse(linebuf, scanpos, &bp, &ep)
+			    && (ep - bp - 1) > strlen(parser[n].needle)) {
+					ADD_TXT_POS(bp, ep, last_index);
+					walk = ep;
+			} else
+				walk = scanpos +
+					strlen(parser[last_index].needle);
+		} else
+			break;
+	}
+
+	/* colorize this line */
+	if (head.next) {
+		const gchar *normal_text = linebuf;
+
+		/* insert URIs */
+		for (last = head.next; last != NULL;
+		     normal_text = last->ep, last = last->next) {
+			RemoteURI *uri;
+
+			uri = g_new(RemoteURI, 1);
+			if (last->bp - normal_text > 0)
+				gtk_text_insert(text, textview->msgfont,
+						fg_color, NULL,
+						normal_text,
+						last->bp - normal_text);
+			uri->uri = parser[last->pti].build_uri(last->bp, last->ep);
+			uri->start = gtk_text_get_point(text);
+			gtk_text_insert(text, textview->msgfont, uri_color,
+					NULL, last->bp, last->ep - last->bp);
+			uri->end = gtk_text_get_point(text);
+			textview->uri_list =
+				g_slist_append(textview->uri_list, uri);
+		}
+
+		if (*normal_text)
+			gtk_text_insert(text, textview->msgfont, fg_color,
+					NULL, normal_text, -1);
+	} else
+		gtk_text_insert(text, textview->msgfont, fg_color, NULL,
+				linebuf, -1);
+}
+
+#undef ADD_TXT_POS
+
 static void textview_write_line(TextView *textview, const gchar *str,
-				const gchar *charset)
-#else
-static void textview_write_line(TextView *textview, const gchar *str,
-				CodeConvFunc code_conv)
-#endif
+				CodeConverter *conv)
 {
 	GtkText *text = GTK_TEXT(textview->text);
 	gchar buf[BUFFSIZE];
 	size_t len;
 	gchar *p;
 	GdkColor *fg_color;
-#if HAVE_LIBJCONV
-	gchar *cstr;
 
-	if ((cstr = conv_codeset_strdup(str, charset, NULL)) == NULL) {
+	if (!conv)
+		strncpy2(buf, str, sizeof(buf));
+	else if (conv_convert(conv, buf, sizeof(buf), str) < 0) {
 		gtk_text_insert(text, textview->msgfont,
 				prefs_common.enable_color
 				? &error_color : NULL, NULL,
-				_("*** Warning: code conversion failed ***\n"),
+				"*** Warning: code conversion failed ***\n",
 				-1);
 		return;
 	}
 
-	strncpy2(buf, cstr, sizeof(buf));
-	g_free(cstr);
-#else /* !HAVE_LIBJCONV */
-	code_conv(buf, sizeof(buf), str);
-#endif
-
 	len = strlen(buf);
 	if (len > 1 && buf[len - 1] == '\n' && buf[len - 2] == '\r') {
 		buf[len - 2] = '\n';
@@ -452,36 +676,10 @@ static void textview_write_line(TextView *textview, const gchar *str,
 	if (prefs_common.head_space && spacingfont && buf[0] != '\n')
 		gtk_text_insert(text, spacingfont, NULL, NULL, " ", 1);
 
-	/* parse URI */
-	if (prefs_common.enable_color &&
-	    ((p = strcasestr(buf, "http://"))  != NULL ||
-	     (p = strcasestr(buf, "https://")) != NULL ||
-	     (p = strcasestr(buf, "ftp://"))   != NULL)) {
-		gchar *ep;
-		RemoteURI *uri;
-
-		uri = g_new(RemoteURI, 1);
-
-		gtk_text_insert(text, textview->msgfont, fg_color, NULL,
-				buf, p - buf);
-
-		/* find end point of URI */
-		for (ep = p; *ep; ep++)
-			if (!isgraph(*ep) || !isascii(*ep) ||
-			    strchr("()<>\"", *ep))
-				break;
-
-		uri->uri = g_strndup(p, ep - p);
-		uri->start = gtk_text_get_point(text);
-		gtk_text_insert(text, textview->msgfont, &uri_color, NULL,
-				p, ep - p);
-		uri->end = gtk_text_get_point(text);
-
-		textview->uri_list = g_slist_append(textview->uri_list, uri);
-
-		gtk_text_insert(text, textview->msgfont, fg_color, NULL,
-				ep, -1);
-	} else
+	if (prefs_common.enable_color)
+		textview_make_clickable_parts(textview, text,
+					      fg_color, &uri_color, buf);
+	else
 		gtk_text_insert(text, textview->msgfont, fg_color, NULL,
 				buf, -1);
 }
@@ -527,6 +725,8 @@ void textview_set_font(TextView *textview, const gchar *codeset)
 {
 	gboolean use_fontset = TRUE;
 
+	/* In multi-byte mode, GtkText can't display 8bit characters
+	   correctly, so it must be single-byte mode. */
 	if (MB_CUR_MAX > 1) {
 		if (codeset) {
 			if (!strncasecmp(codeset, "ISO-8859-", 9) ||
@@ -776,7 +976,7 @@ static void textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	case GDK_End:
 	case GDK_Control_L:
 	case GDK_Control_R:
-		return;
+		break;
 	case GDK_space:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
@@ -796,9 +996,6 @@ static void textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 			summary_pass_key_press_event(summaryview, event);
 		break;
 	}
-
-	if (summaryview)
-		gtk_widget_grab_focus(summaryview->ctree);
 }
 
 static void textview_button_pressed(GtkWidget *widget, GdkEventButton *event,
@@ -808,14 +1005,22 @@ static void textview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 	    ((event->button == 1 && event->type == GDK_2BUTTON_PRESS) ||
 	     event->button == 2)) {
 		GSList *cur;
-		guint current_pos = GTK_EDITABLE(textview->text)->current_pos;
+		guint current_pos;
+
+		current_pos = GTK_EDITABLE(textview->text)->current_pos;
 
 		for (cur = textview->uri_list; cur != NULL; cur = cur->next) {
 			RemoteURI *uri = (RemoteURI *)cur->data;
 
 			if (current_pos >= uri->start &&
-			    current_pos < uri->end)
-				open_uri(uri->uri, prefs_common.uri_cmd);
+			    current_pos <  uri->end) {
+				if (!strncasecmp(uri->uri, "mailto:", 7))
+					compose_new_with_recipient
+						(NULL, uri->uri + 7);
+				else
+					open_uri(uri->uri,
+						 prefs_common.uri_cmd);
+			}				
 		}
 	}
 }
