@@ -841,6 +841,94 @@ GList *uri_list_extract_filenames(const gchar *uri_list)
 	return result;
 }
 
+gchar *strstr_with_skip_quote(const gchar *haystack, const gchar *needle)
+{
+	register guint haystack_len, needle_len;
+	gboolean in_squote = FALSE, in_dquote = FALSE;
+
+	haystack_len = strlen(haystack);
+	needle_len   = strlen(needle);
+
+	if (haystack_len < needle_len || needle_len == 0)
+		return NULL;
+
+	while (haystack_len >= needle_len) {
+		if (!in_squote && !in_dquote &&
+		    !strncmp(haystack, needle, needle_len))
+			return (gchar *)haystack;
+
+		/* 'foo"bar"' -> foo"bar"
+		   "foo'bar'" -> foo'bar' */
+		if (*haystack == '\'') {
+			if (in_squote)
+				in_squote = FALSE;
+			else if (!in_dquote)
+				in_squote = TRUE;
+		} else if (*haystack == '\"') {
+			if (in_dquote)
+				in_dquote = FALSE;
+			else if (!in_squote)
+				in_dquote = TRUE;
+		}
+
+		haystack++;
+		haystack_len--;
+	}
+
+	return NULL;
+}
+
+/* this fuction was taken from gstrfuncs.c in glib. */
+gchar **strsplit_with_quote(const gchar *str, const gchar *delim,
+			    gint max_tokens)
+{
+	GSList *string_list = NULL, *slist;
+	gchar **str_array, *s;
+	guint i, n = 1;
+
+	g_return_val_if_fail(str != NULL, NULL);
+	g_return_val_if_fail(delim != NULL, NULL);
+
+	if (max_tokens < 1)
+		max_tokens = G_MAXINT;
+
+	s = strstr_with_skip_quote(str, delim);
+	if (s) {
+		guint delimiter_len = strlen(delim);
+
+		do {
+			guint len;
+			gchar *new_str;
+
+			len = s - str;
+			new_str = g_new(gchar, len + 1);
+			strncpy(new_str, str, len);
+			new_str[len] = 0;
+			string_list = g_slist_prepend(string_list, new_str);
+			n++;
+			str = s + delimiter_len;
+			s = strstr_with_skip_quote(str, delim);
+		} while (--max_tokens && s);
+	}
+
+	if (*str) {
+		n++;
+		string_list = g_slist_prepend(string_list, g_strdup(str));
+	}
+
+	str_array = g_new(gchar*, n);
+
+	i = n - 1;
+
+	str_array[i--] = NULL;
+	for (slist = string_list; slist; slist = slist->next)
+		str_array[i--] = slist->data;
+
+	g_slist_free(string_list);
+
+	return str_array;
+}
+
 gchar *get_rc_dir(void)
 {
 	static gchar *rc_dir = NULL;
@@ -926,7 +1014,7 @@ off_t get_file_size(const gchar *file)
 	return s.st_size;
 }
 
-gboolean is_file_exist(const gchar *file)
+gboolean file_exist(const gchar *file, gboolean allow_fifo)
 {
 	struct stat s;
 
@@ -935,7 +1023,7 @@ gboolean is_file_exist(const gchar *file)
 		return FALSE;
 	}
 
-	if (S_ISREG(s.st_mode))
+	if (S_ISREG(s.st_mode) || (allow_fifo && S_ISFIFO(s.st_mode)))
 		return TRUE;
 
 	return FALSE;
@@ -1173,7 +1261,7 @@ gint copy_file(const gchar *src, const gchar *dest)
 		return -1;
 	}
 
-	if (chmod(dest, S_IRUSR|S_IWUSR) < 0) {
+	if (change_file_mode_rw(dest_fp, dest) < 0) {
 		FILE_OP_ERROR(dest, "chmod");
 		g_warning(_("can't change file mode\n"));
 	}
@@ -1209,6 +1297,55 @@ gint copy_file(const gchar *src, const gchar *dest)
 	return 0;
 }
 
+gint change_file_mode_rw(FILE *fp, const gchar *file)
+{
+#if HAVE_FCHMOD
+	return fchmod(fileno(fp), S_IRUSR|S_IWUSR);
+#else
+	return chmod(file, S_IRUSR|S_IWUSR);
+#endif
+}
+
+FILE *my_tmpfile(void)
+{
+#if HAVE_MKSTEMP
+	const gchar suffix[] = ".XXXXXX";
+	const gchar *tmpdir;
+	guint tmplen;
+	const gchar *progname;
+	guint proglen;
+	gchar *fname;
+	gint fd;
+	FILE *fp;
+
+	tmpdir = g_get_tmp_dir();
+	tmplen = strlen(tmpdir);
+	progname = g_get_prgname();
+	proglen = strlen(progname);
+	Xalloca(fname, tmplen + 1 + proglen + sizeof(suffix),
+		return tmpfile());
+
+	memcpy(fname, tmpdir, tmplen);
+	fname[tmplen] = G_DIR_SEPARATOR;
+	memcpy(fname + tmplen + 1, progname, proglen);
+	memcpy(fname + tmplen + 1 + proglen, suffix, sizeof(suffix));
+
+	fd = mkstemp(fname);
+	if (fd < 0)
+		return tmpfile();
+
+	unlink(fname);
+
+	fp = fdopen(fd, "w+b");
+	if (!fp)
+		close(fd);
+	else
+		return fp;
+#endif /* HAVE_MKSTEMP */
+
+	return tmpfile();
+}
+
 gint execute_async(gchar *const argv[])
 {
 	pid_t pid;
@@ -1247,7 +1384,7 @@ gint execute_command_line(const gchar *cmdline)
 	gint i;
 	gint ret;
 
-	argv = g_strsplit(cmdline, " ", strlen(cmdline));
+	argv = strsplit_with_quote(cmdline, " ", 0);
 
 	for (i = 0; argv[i] != NULL; i++) {
 		gchar *str = argv[i];
@@ -1258,7 +1395,7 @@ gint execute_command_line(const gchar *cmdline)
 			len = strlen(str);
 			if (str[len - 1] == str[0]) {
 				str[len - 1] = '\0';
-				memmove(str, str + 1, len);
+				memmove(str, str + 1, len - 1);
 			}
 		}
 	}
